==> kube-integration-1.0.11.tgz: package/lib/internal-components/project/controller.js <==
'use strict';

const _       = require('lodash');
const Promise = require('bluebird');
const Project = require('./Project');

class ProjectController {

    getAll({ kubeManager, ns }) {

        const serviceController = kubeManager.getServiceController.bind(kubeManager);
        return serviceController(ns)
            .get()
            .map((service) => {
                const project = new Project({ ns, cluster: kubeManager.getName() });
                project.setService(service);
                const serviceSelector = service.getServiceSelector();
                if (!_.isEmpty(serviceSelector)) {
                    return Promise.map(_.keys(serviceSelector), (key) => {
                        return `${key}=${serviceSelector[key]}`;
                    })
                        .then(_.head) // should support more then one selector
                        .then(this._aggregateResourcesForServiceSelector({
                            kubeManager,
                            ns
                        }))
                        .then(this._updateProjectWithResources(project));
                } else {
                    return project;
                }
            });
    }

    describe({ kubeManager, ns, name }) {
        const serviceController = kubeManager.getServiceController.bind(kubeManager);
        const project           = new Project({ ns, cluster: kubeManager.getName() });
        return serviceController(ns)
            .describe(name)
            .then((service) => {
                project.setService(service);
                const serviceSelector = service.getServiceSelector();
                if (!_.isEmpty(serviceSelector)) {
                    return Promise.map(_.keys(serviceSelector), (key) => {
                        return `${key}=${serviceSelector[key]}`;
                    })
                        .then(_.head) // support more then one selector
                        .then(this._aggregateResourcesForServiceSelector({
                            kubeManager,
                            ns
                        }))
                        .then(this._updateProjectWithResources(project));

                } else {
                    return project;
                }
            });
    }

    create({ kubeManager, ns, data }) {
        const project              = new Project({ ns, cluster: kubeManager.getName() });
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const resourceLogicMapping = {
            service: {
                ctrl: serviceController,
                finally: 'setService'
            },
            deployment: {
                ctrl: deploymentController,
                finally: 'setDeployment'
            },
        };

        return Promise.map(_.keys(data), (key) => {
            if (!resourceLogicMapping[key]) {
                throw new Error(`Cannot create resource ${key}`);
            }
            const value = data[key];
            if (_.isArray(value)) {
                return Promise.map(value,
                    (updateObject) => resourceLogicMapping[key].ctrl(ns).create(updateObject))
                    .then(res => project[resourceLogicMapping[key].finally](res));
            } else {
                return resourceLogicMapping[key].ctrl(ns).create(value)
                    .then(res => project[resourceLogicMapping[key].finally](res));
            }
        })
            .return(project);
    }

    update({ kubeManager, ns, name, data }) {
        const project              = new Project({ ns, cluster: kubeManager.getName() });
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const resourceMapping      = {
            service: {
                ctrl: serviceController,
                finally: 'setService'
            },
            deployment: {
                ctrl: deploymentController,
                finally: 'setDeployment'
            },
        };

        return Promise.map(_.keys(data), (key) => {
            if (!resourceMapping[key]) {
                throw new Error(`Cannot update resource ${key}`);
            }
            const value = data[key];
            if (_.isArray(value)) {
                return Promise.map(value,
                    (updateObject) => resourceMapping[key].ctrl(ns).update(name, updateObject))
                    .then(res => project[resourceMapping[key].finally](res));
            } else {
                return resourceMapping[key].ctrl(ns).update(name, value)
                    .then(res => project[resourceMapping[key].finally](res));
            }
        })
            .return(project);
    }

    delete({ kubeManager, project }) {
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const candidate            = project.getName();
        const ns                   = project.getNamespace();

        return Promise.resolve()
            .then(() =>
                serviceController(ns).delete(candidate)
            )
            .then((selector) =>
                // what if the deployment name is not the same ??
                deploymentController(ns)
                    .delete(candidate)
            );
    }

    _aggregateResourcesForServiceSelector({ kubeManager, ns }) {
        return function (selector) {
            const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
            const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
            const podController        = kubeManager.getPodController.bind(kubeManager);

            const allDeploymentsPromise = deploymentController(ns)
                .get({ labelSelector: selector })
                .then(_.head);
            const allPodsPromise        = podController(ns).get({ labelSelector: selector });
            const allRsPromise          = replicasetController(ns).get({ labelSelector: selector })
                .map(rs => rs.getReplicas() !== 0 ? rs : undefined)
                .then(_.compact);
            return Promise.all([allDeploymentsPromise,
                allRsPromise,
                allPodsPromise
            ]);
        }
    }

    _updateProjectWithResources(project) {
        return function ([deployment, rsArray, podArray]) {
            deployment && project.setDeployment(deployment);
            _.size(rsArray) > 0 && project.setReplicaSets(rsArray);
            _.size(podArray) > 0 && project.setPods(podArray);
            return project;
        }
    }

}

module.exports = new ProjectController();

==> kube-integration-1.21.0.tgz: package/lib/internal-components/project/controller.js <==
'use strict';

const _       = require('lodash');
const Promise = require('bluebird');
const Project = require('./Project');
const CFError = require('cf-errors');

class ProjectController {

    getAll({ kubeManager, ns }) {

        const serviceController = kubeManager.getServiceController.bind(kubeManager);
        return serviceController(ns)
            .get()
            .map((service) => {
                const project = new Project({ ns, cluster: kubeManager.getName() });
                project.setService(service);
                const serviceSelector = service.getServiceSelector();
                if (!_.isEmpty(serviceSelector)) {
                    return Promise.map(_.keys(serviceSelector), (key) => {
                        return `${key}=${serviceSelector[key]}`;
                    })
                        .then(_.head) // should support more then one selector
                        .then(this._aggregateResourcesForServiceSelector({
                            kubeManager,
                            ns
                        }))
                        .then(this._updateProjectWithResources(project));
                } else {
                    return project;
                }
            });
    }

    describe({ kubeManager, ns, name }) {
        const serviceController = kubeManager.getServiceController.bind(kubeManager);
        const project           = new Project({ ns, cluster: kubeManager.getName() });
        return serviceController(ns)
            .describe(name)
            .then((service) => {
                project.setService(service);
                const serviceSelector = service.getServiceSelector();
                if (!_.isEmpty(serviceSelector)) {
                    return Promise.map(_.keys(serviceSelector), (key) => {
                        return `${key}=${serviceSelector[key]}`;
                    })
                        .then(_.head) // support more then one selector
                        .then(this._aggregateResourcesForServiceSelector({
                            kubeManager,
                            ns
                        }))
                        .then(this._updateProjectWithResources(project));

                } else {
                    return project;
                }
            });
    }

    create({ kubeManager, ns, data }) {
        const project              = new Project({ ns, cluster: kubeManager.getName() });
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const resourceLogicMapping = {
            service: {
                ctrl: serviceController,
                finally: 'setService'
            },
            deployment: {
                ctrl: deploymentController,
                finally: 'setDeployment'
            },
        };

        return Promise.map(_.keys(data), (key) => {
            if (!resourceLogicMapping[key]) {
                throw new CFError(`Cannot create resource ${key}`);
            }
            const value = data[key];
            if (_.isArray(value)) {
                return Promise.map(value,
                    (updateObject) => resourceLogicMapping[key].ctrl(ns).create(updateObject))
                    .then(res => project[resourceLogicMapping[key].finally](res));
            } else {
                return resourceLogicMapping[key].ctrl(ns).create(value)
                    .then(res => project[resourceLogicMapping[key].finally](res));
            }
        })
            .return(project);
    }

    update({ kubeManager, ns, name, data }) {
        const project              = new Project({ ns, cluster: kubeManager.getName() });
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const resourceMapping      = {
            service: {
                ctrl: serviceController,
                finally: 'setService'
            },
            deployment: {
                ctrl: deploymentController,
                finally: 'setDeployment'
            },
        };

        return Promise.map(_.keys(data), (key) => {
            if (!resourceMapping[key]) {
                throw new CFError(`Cannot update resource ${key}`);
            }
            const value = data[key];
            if (_.isArray(value)) {
                return Promise.map(value,
                    (updateObject) => resourceMapping[key].ctrl(ns).update(name, updateObject))
                    .then(res => project[resourceMapping[key].finally](res));
            } else {
                return resourceMapping[key].ctrl(ns).update(name, value)
                    .then(res => project[resourceMapping[key].finally](res));
            }
        })
            .return(project);
    }

    delete({ kubeManager, project }) {
        const serviceController    = kubeManager.getServiceController.bind(kubeManager);
        const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
        const candidate            = project.getName();
        const ns                   = project.getNamespace();

        return Promise.resolve()
            .then(() =>
                serviceController(ns).delete(candidate)
            )
            .then((selector) =>
                // what if the deployment name is not the same ??
                deploymentController(ns)
                    .delete(candidate)
            );
    }

    _aggregateResourcesForServiceSelector({ kubeManager, ns }) {
        return function (selector) {
            const deploymentController = kubeManager.getDeploymentController.bind(kubeManager);
            const replicasetController = kubeManager.getReplicasetController.bind(kubeManager);
            const podController        = kubeManager.getPodController.bind(kubeManager);

            const allDeploymentsPromise = deploymentController(ns)
                .get({ labelSelector: selector })
                .then(_.head);
            const allPodsPromise        = podController(ns).get({ labelSelector: selector });
            const allRsPromise          = replicasetController(ns).get({ labelSelector: selector })
                .map(rs => rs.getReplicas() !== 0 ? rs : undefined)
                .then(_.compact);
            return Promise.all([allDeploymentsPromise,
                allRsPromise,
                allPodsPromise
            ]);
        }
    }

    _updateProjectWithResources(project) {
        return function ([deployment, rsArray, podArray]) {
            deployment && project.setDeployment(deployment);
            _.size(rsArray) > 0 && project.setReplicaSets(rsArray);
            _.size(podArray) > 0 && project.setPods(podArray);
            return project;
        }
    }

}

module.exports = new ProjectController();
